<?php

namespace FiloBlu\AdyenErrorNotificationHandler\Observer;

use FiloBlu\AdyenErrorNotificationHandler\Helper\Data;
use Magento\Framework\DataObject;

class ErrorNotify implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var Data
     */
    protected $dataHelper;

    public function __construct(Data $dataHelper){
        $this->dataHelper = $dataHelper;
    }

    /**
     * Execute observer
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(
        \Magento\Framework\Event\Observer $observer
    ) {
        if($this->dataHelper->adyenErrorNotificationHandlerIsEnabled())
        {
            $response = $observer->getEvent()->getResponse();
            $language = $observer->getEvent()->getLanguage();
            $is3dsV1 = $observer->getEvent()->getSecureCodeOne();

            $errorMessages = json_decode($this->dataHelper->getAdyenErrorNotificationMessages(), true);

            $dataToLog = array(
                'response' => $response,
                'lang' => $language,
                '3ds_v1' => $is3dsV1
            );

            if(!is_null($errorMessages) && is_array($errorMessages))
            {
                $errorInLang = $this->dataHelper->getErrorsInLanguage($errorMessages, $language);

                if(is_null($errorInLang) && $this->dataHelper->useErrorMessagesInDefaultLangEnabled())
                {
                    /*
                     * If the messages translations are missing, I search the default language translations.
                     */
                    $defaultLang = $this->dataHelper->getDefaultLang();
                    $errorInLang = $this->dataHelper->getErrorsInLanguage($errorMessages, $defaultLang);
                }

                if(!is_null($errorInLang))
                {
                    $errorArray = array();
                    if(!$is3dsV1)
                    {
                        $error = $this->dataHelper->getErrorByCode($errorInLang, $response[$this->dataHelper::ADYEN_REFUSAL_REASON_CODE]);

                        if(!is_null($error))
                        {
                            $dataToLog['module_error'] = $error;

                            /**
                             * @var DataObject
                             */
                            $checkMappedMessage = $observer->getEvent()->getMappedErrorMessage();
                            if(is_null($checkMappedMessage) || !$checkMappedMessage)
                            {
                                $this->dataHelper->manageDataToLog($dataToLog);
                                throw new \Magento\Framework\Exception\LocalizedException(__($error));
                            }
                            else
                            {
                                $errorArray[] = $error;
                                $checkMappedMessage->setData('mapped_error', $errorArray);
                            }
                        }
                    }
                    else
                    {
                        $data = $observer->getEvent()->getData();
                        $messageManager = $data['messageManager'];

                        $codeToCheck = $response[$this->dataHelper::ADYEN_REFUSAL_REASON_CODE] != 0 ? $response[$this->dataHelper::ADYEN_REFUSAL_REASON_CODE] : $this->dataHelper::ERROR_CODE_3DV1;
                        $error = $this->dataHelper->getErrorByCode($errorInLang, $codeToCheck);
                        if(!is_null($error))
                        {
                            $dataToLog['module_error'] = $error;
                            $messageManager->addErrorMessage($error);
                        }
                        else
                        {
                            $messageManager->addErrorMessage("3D-secure validation was unsuccessful");
                        }
                    }
                }
            }

            $this->dataHelper->manageDataToLog($dataToLog);
        }
    }
}